#!/bin/sh
#
# mkrevision.sh - generate revision headers from Git and Autoconf metadata
#
# Copyright (C) 2006-2007 Carnegie Mellon University
#
# This software is distributed under the terms of the Eclipse Public
# License, Version 1.0 which can be found in the file named LICENSE.Eclipse.
# ANY USE, REPRODUCTION OR DISTRIBUTION OF THIS SOFTWARE CONSTITUTES
# RECIPIENT'S ACCEPTANCE OF THIS AGREEMENT
#

set -e

FILETYPE=$1
BASEDIR=`dirname $0`

if [ -d $BASEDIR/.git ] ; then
	REV=`git-describe`
	if git-diff-index HEAD | read junk ; then
		# There are uncommitted changes in the working copy
		REV="$REV-dirty"
	fi
	echo $REV > $BASEDIR/.gitrevision
else
	REV=`cat $BASEDIR/.gitrevision`
fi
REL=`awk 'BEGIN { FS="\"" } /#define PACKAGE_VERSION/ { print $2 }' \
			$BASEDIR/config.h` 

# It's better to use a separate object file for the revision data,
# since "svn update" will then force a relink but not a recompile.
# However, we shouldn't do this for shared libraries, because then
# "rcs_revision" becomes part of the library's ABI.
case $FILETYPE in
object)
	FILENAME=revision.c
	cat > $FILENAME-new <<- EOF
		const char *isr_release = "$REL";
		const char *rcs_revision = "$REV";
		
		#ifdef __KERNEL__
		#include <linux/module.h>
		MODULE_VERSION("$REL");
		MODULE_INFO(revision, "$REV");
		#endif
	EOF
	;;
header)
	FILENAME=revision.h
	cat > $FILENAME-new <<- EOF
		#define ISR_RELEASE "$REL"
		#define RCS_REVISION "$REV"
	EOF
	;;
perl)
	FILENAME=IsrRevision.pm
	cat > $FILENAME-new <<- EOF
		package Isr;
		\$ISR_RELEASE = "$REL";
		\$RCS_REVISION = "$REV";
		1;
	EOF
	;;
none)
	# Just generate .gitrevision
	exit 0
	;;
*)
	echo "Usage: $0 {object|header|perl|none} <subdir>" >&2
	exit 1
	;;
esac

if [ -f $FILENAME ] && cmp -s $FILENAME $FILENAME-new ; then
	# No need to rebuild if the actual content of $FILENAME
	# hasn't changed
	rm -f $FILENAME-new
else
	mv $FILENAME-new $FILENAME
fi
